#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Python PWRCheck Reader Constants."""

__author__ = 'Greg Albrecht W2GMD <oss@undef.net>'
__copyright__ = 'Copyright 2018 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


class ParseError(ValueError):
    def __init__(self, message, data):
        super(ParseError, self).__init__((message, data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Python PWRCheck Reader Class Definitions."""

import logging
import threading

import serial

import pwrcheck

__author__ = 'Greg Albrecht W2GMD <oss@undef.net>'
__copyright__ = 'Copyright 2018 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


class SerialPoller(threading.Thread):

    """Threadable Object for polling a Serial Device."""

    _logger = logging.getLogger(__name__)
    if not _logger.handlers:
        _logger.setLevel(pwrcheck.LOG_LEVEL)
        _console_handler = logging.StreamHandler()
        _console_handler.setLevel(pwrcheck.LOG_LEVEL)
        _console_handler.setFormatter(pwrcheck.LOG_FORMAT)
        _logger.addHandler(_console_handler)
        _logger.propagate = False

    def __init__(self, serial_port, serial_speed):
        threading.Thread.__init__(self)
        self._serial_port = serial_port
        self._serial_speed = serial_speed
        self._stopped = False

        self.pwrcheck_props = {}
        for prop in pwrcheck.PWRCHECK_PROPERTIES:
            self.pwrcheck_props[prop] = None

        self._serial_int = serial.Serial(
            self._serial_port, self._serial_speed, timeout=1)

    def stop(self):
        """
        Stop the thread at the next opportunity.
        """
        self._stopped = True
        return self._stopped

    def run(self):
        streamreader = StreamReader(self._serial_int)
        try:
            while not self._stopped:
                for msg in streamreader.next():
                    for prop in pwrcheck.PWRCHECK_PROPERTIES:
                        if getattr(msg, prop, None) is not None:
                            self.pwrcheck_props[prop] = getattr(msg, prop)
                            # self._logger.debug(
                            #    '%s=%s', prop, self.pwrcheck_props[prop])
        except StopIteration:
            pass


class StreamReader(object):
    """
    Reads Lines from a stream.
    """

    def __init__(self, stream=None, errors='raise'):
        """
        `stream`:   file-like object to read from, can be omitted to
                    pass data to `next` manually.
                    must support `.readline()` which returns a string
        `errors`: behaviour when a parse error is encountered. can be one of:
            `'raise'` (default) raise an exception immediately
            `'yield'`           yield the ParseError as an element in the
                                stream, and continue reading at the next line
            `'ignore'`          completely ignore and suppress the error, and
                                continue reading at the next line
        """

        if errors not in pwrcheck.STREAM_ERRORS:
            raise ValueError(
                'errors must be one of {!r} (was: {!r})'.format(
                    pwrcheck.STREAM_ERRORS, errors))

        self.errors = errors
        self.stream = stream
        self.buffer = b''

    def next(self, data=None):
        """
        consume `data` (if given, or calls `stream.read()` if `stream` was
        given in the constructor) and yield a list of `NMEASentence` objects
        parsed from the stream (may be empty)
        """
        if data is None:
            if self.stream:
                data = self.stream.readline()
            else:
                return

        lines = (self.buffer + data).split(b'\n')
        self.buffer = lines.pop()

        for line in lines:
            try:
                yield line
            except pwrcheck.ParseError as exc:
                if self.errors == 'raise':
                    raise exc
                if self.errors == 'yield':
                    yield exc
                if self.errors == 'ignore':
                    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Python PWRCheck Reader Commands."""

import argparse
import time

import pwrcheck

__author__ = 'Greg Albrecht W2GMD <oss@undef.net>'
__copyright__ = 'Copyright 2018 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


def cli():
    """Tracker Command Line interface for PWRCheck."""

    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-s', '--serial_port', help='serial_port', required=True
    )
    parser.add_argument(
        '-b', '--serial_speed', help='serial_speed', default=115200
    )
    parser.add_argument(
        '-i', '--interval', help='interval', default=0, type=int
    )

    opts = parser.parse_args()

    pwrcheck_poller = pwrcheck.SerialPoller(
        opts.serial_port, opts.serial_speed)
    pwrcheck_poller.start()

    time.sleep(pwrcheck.PWRCHECK_WARM_UP)

    import pprint

    try:
        while 1:
            pprint.pprint(pwrcheck_poller.pwrcheck_props)

            if opts.interval == 0:
                break
            else:
                time.sleep(opts.interval)

    except KeyboardInterrupt:
        pwrcheck_poller.stop()
    finally:
        pwrcheck_poller.stop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Python PWRCheck Reader.

"""
Python PWRCheck Reader.
~~~~


:author: Greg Albrecht W2GMD <oss@undef.net>
:copyright: Copyright 2018 Greg Albrecht
:license: Apache License, Version 2.0
:source: <https://github.com/ampledata/pwrcheck>

"""

from .exceptions import ParseError  # NOQA

from .constants import (LOG_FORMAT, LOG_LEVEL, PWRCHECK_PROPERTIES,  # NOQA
                        PWRCHECK_WARM_UP, STREAM_ERRORS)

from .classes import SerialPoller  # NOQA

__author__ = 'Greg Albrecht W2GMD <oss@undef.net>'
__copyright__ = 'Copyright 2018 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Python PWRCheck Reader Constants."""

import logging

__author__ = 'Greg Albrecht W2GMD <oss@undef.net>'
__copyright__ = 'Copyright 2018 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


LOG_LEVEL = logging.DEBUG
LOG_FORMAT = logging.Formatter(
    ('%(asctime)s pwrcheck %(levelname)s %(name)s.%(funcName)s:%(lineno)d '
     ' - %(message)s'))

STREAM_ERRORS = ('raise', 'yield', 'ignore')

PWRCHECK_WARM_UP = 5

PWRCHECK_PROPERTIES = [
    'volts',
    'amps',
    'watts',
    'total_amp_hours',
    'forward_amp_hours',
    'reverse_amp_hours',
    'max_volts',
    'min_volts',
    'max_amps',
    'min_amps',
    'quality'
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Setup for the Python PWRCheck Reader.

Source:: https://github.com/ampledata/pwrcheck
"""

import os
import sys

import setuptools

__title__ = 'pwrcheck'
__version__ = '0.0.1b2'
__author__ = 'Greg Albrecht W2GMD <oss@undef.net>'
__copyright__ = 'Copyright 2018 Greg Albrecht'
__license__ = 'Apache License, Version 2.0'


def publish():
    """Function for publishing package to pypi."""
    if sys.argv[-1] == 'publish':
        os.system('python setup.py sdist')
        os.system('twine upload dist/*')
        sys.exit()


publish()


setuptools.setup(
    name=__title__,
    version=__version__,
    description='Python PWRCheck Reader.',
    author='Greg Albrecht',
    author_email='oss@undef.net',
    packages=['pwrcheck'],
    package_data={'': ['LICENSE']},
    package_dir={'pwrcheck': 'pwrcheck'},
    license=open('LICENSE').read(),
    long_description=open('README.rst').read(),
    url='https://github.com/ampledata/pwrcheck',
    zip_safe=False,
    include_package_data=True,
    setup_requires=[
        'coverage >= 3.7.1',
        'httpretty >= 0.8.10',
        'nose >= 1.3.7'
    ],
    install_requires=[
        'pynmea2 >= 1.4.2',
        'pyserial >= 2.7',
    ],
    classifiers=[
        'Topic :: Communications :: Ham Radio',
        'Programming Language :: Python',
        'License :: OSI Approved :: Apache Software License'
    ],
    keywords=[
        'Ham Radio', 'APRS', 'GPS'
    ],
    entry_points={'console_scripts': ['pwrcheck = pwrcheck.cmd:cli']}
)
